<?php

namespace NoccyLabs\TinyData\Collection;

use NoccyLabs\TinyData\DataSource;
use NoccyLabs\TinyData\Schema\Schema;
use PDO;

/**
 * Hybrid storage type that stores documents (array structures) as serialized
 * blobs after extracting indexable fields to their own columns. Data can be
 * found based on the indexed columns.
 *
 */
class DocumentCollection extends Collection
{
    /**
     * Insert a new document
     *
     */
    public function insert(array $document)
    {
        // extract indexes
        $cols = []; $vals = [];
        foreach ($this->schema->indexed_fields as $field) {
            $fieldval = $this->getValueByPath($document, $field->path, $field->type);
            $cols[] = $field->alias;
            $cstr[] = ":".$field->alias;
            $vals[":".$field->alias] = $fieldval;
        }

        $cols[] = "document"; $cstr[] = ":document";
        $stmt = sprintf("INSERT INTO {$this->name} (%s) VALUES (%s)", join(",",$cols), join(",",$cstr));
        $data = array_merge($vals, [ ':document' => serialize($document) ]);
        $this->ds->query($stmt, $data);
    }


    public function update(array $document, array $match)
    {
        
    }

    /**
     * Delete a document
     *
     */
    public function delete(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("DELETE FROM {$this->name} WHERE %s", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        return $ret;
    }

    /**
     * Delete all documents
     *
     */
    public function deleteAll()
    {
        $stmt = "DELETE FROM {$this->name} WHERE 1==1";
        $this->ds->execute($stmt);
    }

    /**
     * Find all matching documents
     *
     */
    public function find(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("SELECT ROWID,* FROM {$this->name} WHERE %s", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        if (count($ret)==0) return null;

        $docs = [];

        foreach ($ret as $match) {
            $doc = (array)unserialize($match['document']);
            $doc['_rowid'] = $match['rowid'];
            $docs[] = $doc;
        }

        return $docs;        
    }

    /**
     * Find a single document
     *
     */
    public function findOne(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("SELECT ROWID,* FROM {$this->name} WHERE %s LIMIT 1", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        if (count($ret)==0) return null;

        $doc = (array)unserialize($ret[0]['document']);
        $doc['_rowid'] = $ret[0]['rowid'];
        return $doc;

    }

    /**
     * Find all documents.
     *
     */
    public function findAll()
    {
        $ret = $this->ds->query("SELECT ROWID,document FROM {$this->name}");
        $docs = [];

        foreach ($ret as $match) {
            $doc = (array)unserialize($match['document']);
            $doc['_rowid'] = $match['rowid'];
            $docs[] = $doc;
        }

        return $docs;
        
    }

    private function getValueByPath(array $document, $path, $type)
    {
        $seg = explode("/",trim($path,"/"));
        $ptr = $document;
        foreach ($seg as $key) {
            if (!array_key_exists($key, $ptr)) {
                return null;
            } else {
                $ptr = $ptr[$key];
            }
        }
        switch (strtoupper($type)) {
            case 'INT':
            case 'NUMERIC':
                return @intval($ptr);
            case 'FLOAT':
                return @floatval($ptr);
            default:
                if (is_array($ptr))
                    return null;
                return strval($ptr);
        }
    }


}
