<?php

namespace NoccyLabs\TinyData\Collection;

use NoccyLabs\TinyData\DataSource;
use NoccyLabs\TinyData\Schema\Schema;
use PDO;

class KeyValueCollection extends Collection
{

    public function set($key, $value)
    {
        $param = [ ':key' => $key, ':value' => serialize($value) ];
        $this->ds->query("REPLACE INTO {$this->name} (key,value) VALUES (:key,:value)", $param);
    }

    public function get($key)
    {
        $param = [ ':key' => $key ];
        $result = $this->ds->query("SELECT * FROM {$this->name} WHERE key=:key", $param);
        if (count($result)==0) {
            return null;
        }
        return unserialize($result[0]['value']);
    }

    public function getAll()
    {
        $result = $this->ds->query("SELECT * FROM {$this->name}");
        $kvpairs = [];
        foreach ($result as $row) {
            $kvpairs[$row['key']] = unserialize($row['value']);
        }
        return $kvpairs;
    }

    public function delete($key)
    {
        $param = [ ':key' => $key ];
        $this->ds->query("DELETE FROM {$this->name} WHERE key=:key", $param);
    }

    public function deleteAll()
    {
        $this->ds->execute("DELETE FROM {$this->name} WHERE 1==1");
    }

    public function __get($key)
    {
        return $this->get($key);
    }

    public function __set($key, $value)
    {
        $this->set($key, $value);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }

    public function __isset($key)
    {
        return ($this->get($key) !== null);
    }

}
