<?php

namespace NoccyLabs\TinyData\Collection;

use NoccyLabs\TinyData\DataSource;
use NoccyLabs\TinyData\Schema\Schema;
use PDO;

class TableCollection extends Collection
{

    /**
     *
     *
     * @param array $row The row to insert
     */
    public function insert(array $row)
    {
        $keys = []; $vals = []; $data = [];
        foreach ($row as $k=>$v) {
            $keys[] = $k;
            $vals[] = ":{$k}";
            $data[":{$k}"] = $v;
        }
        $stmt = sprintf("INSERT INTO {$this->name} (%s) VALUES (%s)", join(",",$keys), join(",",$vals));
        $this->ds->query($stmt, $data);
    }

    /**
     *
     *
     * @param array $row The row to insert or replace
     */
    public function replace(array $row)
    {
        $keys = []; $vals = []; $data = [];
        foreach ($row as $k=>$v) {
            $keys[] = $k;
            $vals[] = ":{$k}";
            $data[":{$k}"] = $v;
        }
        $stmt = sprintf("REPLACE INTO {$this->name} (%s) VALUES (%s)", join(",",$keys), join(",",$vals));
        $this->ds->query($stmt, $data);
    }

    /**
     *
     *
     * @param array $match Expression to match against data
     */
    public function delete(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("DELETE FROM {$this->name} WHERE %s", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        return $ret;
    }

    /**
     * Delete all rows from the table
     *
     */
    public function deleteAll()
    {
        $stmt = "DELETE FROM {$this->name} WHERE 1==1";
        $this->ds->execute($stmt);
    }

    /**
     *
     *
     * @param array $row The row to update
     * @param array $match Expression to match against data
     */
    public function update(array $row, array $match)
    {
        $expr = $this->matchToStatement($match);
        $kvs = []; $data = [];
        foreach ($row as $k=>$v) {
            $kvs[] = "$k=:{$k}";
            $data[":{$k}"] = $v;
        }
        $stmt = sprintf("UPDATE {$this->name} SET %s WHERE %s", join(",",$kvs), join(" AND ",$expr));
        $this->ds->query($stmt, $data);
    }

    /**
     *
     * @param array $match Expression to match against data
     * @return array The found rows
     */
    public function find(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("SELECT ROWID,* FROM {$this->name} WHERE %s", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        return $ret;
    }

    /**
     *
     *
     * @param array $match Expression to match against data
     * @return array|null The found row
     */
    public function findOne(array $match)
    {
        $expr = $this->matchToStatement($match);
        $stmt = sprintf("SELECT ROWID,* FROM {$this->name} WHERE %s LIMIT 1", join(" AND ",$expr));
        $ret = $this->ds->query($stmt,[]);
        if (count($ret)==0) return null;
        return $ret[0];
    }

    /**
     * Find all rows in the table.
     *
     * @return array The rows
     */
    public function findAll()
    {
        $ret = $this->ds->query("SELECT ROWID,* FROM {$this->name}");
        return $ret;
    }

}
