<?php

namespace NoccyLabs\TinyData\Schema;

use NoccyLabs\TinyData\Collection\DocumentCollection;

/**
 *
 *
 */
class DocumentSchema extends Schema
{

    public $indexed_fields = [];

    /**
     * Schema constructor
     *
     * @param string $name The schema name
     * @param int $version The schema version
     * @param array $indexed The indexed fields
     * @param array $definition Schema definition
     */
    public function __construct($name, $version, array $indexed=[])
    {
        parent::__construct($name, $version);
        
        $this->setColumn("document");
        foreach ($indexed as $k=>$v) {
            if (is_numeric($k)) {
                $this->setIndexedField($v);
            } else {
                $this->setIndexedField($k, $v);
            }
        }
    }

    public function setIndexedField($path, $alias=null)
    {
        // Check for unique index flag
        $unique = false;
        if (substr($path,-1,1)=="*") {
            $unique = true;
            $path = substr($path,0,-1);
        }

        // Check for type definition
        if (strpos($path,":")!==false) {
            list ($path, $type) = explode(":",$path,2);
        } else {
            $type = "TEXT";
        }

        $alias = $alias?:basename($path);
        // printf("path=%s alias=%s type=%s unique=%d\n", $path, $alias, $type, $unique);
        $this->setColumn($alias, $type);
        $this->setIndex($alias, [ $alias ], $unique);

        $this->indexed_fields[$alias] = (object)[
            'path' => $path,
            'type' => $type,
            'alias' => $alias,
            'unique' => $unique,
        ];
    }

    /**
     * {@inheritDoc}
     */
    public function getCollectionType()
    {
        return DocumentCollection::class;
    }

}
