<?php

use NoccyLabs\TinyData\DataStore;

class DocumentTest extends PhpUnit_Framework_TestCase
{
    public function setup()
    {
        $dbf = DBFILE("document");
        $this->ds = new DataStore($dbf);
    }

    public function teardown()
    {
        $this->ds = null;
        $dbf = DBFILE("document");
        if (file_exists($dbf)) unlink($dbf);
    }

    public function testCreatingSchema()
    {
        $this->ds->createDocumentSchema("testc", 1, [
            "/a/id:INT*",
            "/b/foo" => "foo"
        ], true );
        
        $create = exec("sqlite3 ".DBFILE('document')." .dump | grep 'CREATE TABLE \"testc\"'");
        exec("sqlite3 ".DBFILE('document')." .dump | grep 'INDEX' | grep testc", $indexes);

        $this->assertEquals("CREATE TABLE \"testc\" (document TEXT, id INT, foo TEXT);", $create);
        $this->assertContains("CREATE UNIQUE INDEX idx_testc_id ON \"testc\" (id);", $indexes);
        $this->assertContains("CREATE INDEX idx_testc_foo ON \"testc\" (foo);", $indexes);

    }

    public function testCreateInsert()
    {
        $this->ds->createDocumentSchema("testc", 1, [
            "/a/id:INT*",
            "/b/foo" => "foo"
        ], true );

        $this->ds->testc->insert([
            'a' => [ 'id'=>4 ],
            'b' => [ 'foo'=>'Hello', 'bar'=>'World' ]
        ]);        

        $this->ds->testc->insert([
            'a' => [ 'id'=>6 ],
            'b' => [ 'foo'=>'Test', 'bar'=>'String' ]
        ]);        

        $rec = $this->ds->testc->find([ 'id'=>4 ]);
        $this->assertEquals(count($rec),1);
        
        $rec = $this->ds->testc->find([ 'id'=>8 ]);
        $this->assertEquals(count($rec),0);

        $rec = $this->ds->testc->findAll();
        $this->assertEquals(count($rec),2);
        $this->assertEquals("Hello", $rec[0]['b']['foo']);
        $this->assertEquals("Test", $rec[1]['b']['foo']);
        
    }

}
