<?php

use NoccyLabs\TinyData\DataStore;

class KeyValueTest extends PhpUnit_Framework_TestCase
{
    public function setup()
    {
        $dbf = DBFILE("keyvalue");
        $this->ds = new DataStore($dbf);
    }

    public function teardown()
    {
        $this->ds = null;
        $dbf = DBFILE("keyvalue");
        if (file_exists($dbf)) unlink($dbf);
    }

    public function testCreatingSchema()
    {
        $testb = $this->ds->createKeyValueSchema("testb");
        
        $this->ds->testb = $testb;

        $create = exec("sqlite3 ".DBFILE('keyvalue')." .dump | grep 'CREATE TABLE \"testb\"'");
        $index = exec("sqlite3 ".DBFILE('keyvalue')." .dump | grep 'CREATE UNIQUE INDEX' | grep testb");

        $this->assertEquals("CREATE TABLE \"testb\" (key TEXT NOT NULL, value TEXT);", $create);
        $this->assertEquals("CREATE UNIQUE INDEX idx_testb_key ON \"testb\" (key);", $index);
    }

    public function testSetGet()
    {
        $testb = $this->ds->createKeyValueSchema("testb");
        
        $this->ds->testb = $testb;
        
        $this->ds->testb->set('foo', 'hello');
        $this->ds->testb->bar = 'world';

        $rec = $this->ds->testb->foo;
        $this->assertEquals('hello', $rec);

        $rec = $this->ds->testb->get('bar');
        $this->assertEquals('world', $rec);

    }

    public function testSetDelete()
    {
        $testb = $this->ds->createKeyValueSchema("testb");
        
        $this->ds->testb = $testb;
        
        $this->ds->testb->set('foo', 'hello');
        $this->ds->testb->bar = 'world';

        $this->ds->testb->delete('foo');

        $rec = $this->ds->testb->foo;
        $this->assertNull($rec);

    }

    public function testSetGetDelete()
    {
        $testb = $this->ds->createKeyValueSchema("testb");
        
        $this->ds->testb = $testb;
        
        $this->ds->testb->set('foo', 'hello');
        $this->ds->testb->bar = 'world';

        $rec = $this->ds->testb->getAll();
        $this->assertEquals(2, count($rec));

        $this->ds->testb->deleteAll();

        $rec = $this->ds->testb->getAll();
        $this->assertEquals(0, count($rec));

    }

}
